/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.rest;

import java.util.Properties;

import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.rest.RestAgreement;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.ogf.graap.wsag.client.api.AgreementClient;
import org.ogf.graap.wsag.client.api.RemoteClient;
import org.ogf.graap.wsag.client.api.impl.AgreementClientImpl;
import org.ogf.schemas.graap.wsAgreement.AgreementContextType;
import org.ogf.schemas.graap.wsAgreement.AgreementPropertiesDocument;
import org.ogf.schemas.graap.wsAgreement.AgreementPropertiesType;
import org.ogf.schemas.graap.wsAgreement.AgreementStateType;
import org.ogf.schemas.graap.wsAgreement.GuaranteeTermStateType;
import org.ogf.schemas.graap.wsAgreement.ServiceTermStateType;
import org.ogf.schemas.graap.wsAgreement.TermTreeType;
import org.ogf.schemas.graap.wsAgreement.TerminateInputDocument;
import org.ogf.schemas.graap.wsAgreement.TerminateInputType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * Default implementation of an agreement client.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class RestAgreementClient extends AgreementClientImpl
{

    private final RestRemoteClient<RestAgreement> client;

    /**
     * Instantiates a new agreement client for the given EPR.
     * 
     * @param epr
     *            the agreement endpoint reference.
     * 
     * @param securityProperties
     *            the security properties
     */
    public RestAgreementClient( EndpointReferenceType epr, ISecurityProperties securityProperties )
    {
        client =
            RestRemoteClient.getInstance( epr, new Properties(), securityProperties, RestAgreement.class );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AgreementClient clone() throws CloneNotSupportedException
    {
        return new RestAgreementClient( client.getRemoteReference(), client.getSecurityProperties().clone() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public RemoteClient getRemoteClient()
    {
        return client;
    }

    @Override
    public AgreementContextType getContext() throws ResourceUnknownException, ResourceUnavailableException
    {
        AgreementPropertiesDocument propertiesDocument = client.getApplicationClient().getContext();
        return propertiesDocument.getAgreementProperties().getContext();
    }

    @Override
    public GuaranteeTermStateType[] getGuaranteeTermStates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        AgreementPropertiesDocument propertiesDocument =
            client.getApplicationClient().getGuaranteeTermStates();
        return propertiesDocument.getAgreementProperties().getGuaranteeTermStateArray();
    }

    @Override
    public String getAgreementId() throws ResourceUnknownException, ResourceUnavailableException
    {
        return client.getApplicationClient().getAgreementId();
    }

    @Override
    public String getName() throws ResourceUnknownException, ResourceUnavailableException
    {
        return client.getApplicationClient().getName();
    }

    @Override
    public ServiceTermStateType[] getServiceTermStates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        AgreementPropertiesDocument propertiesDocument = client.getApplicationClient().getServiceTermStates();
        return propertiesDocument.getAgreementProperties().getServiceTermStateArray();
    }

    @Override
    public AgreementStateType getState() throws ResourceUnknownException, ResourceUnavailableException
    {
        AgreementPropertiesDocument propertiesDocument = client.getApplicationClient().getState();
        return propertiesDocument.getAgreementProperties().getAgreementState();
    }

    @Override
    public TermTreeType getTerms() throws ResourceUnknownException, ResourceUnavailableException
    {
        AgreementPropertiesDocument propertiesDocument = client.getApplicationClient().getTerms();
        return propertiesDocument.getAgreementProperties().getTerms();
    }

    @Override
    public void terminate( TerminateInputType reason )
        throws ResourceUnknownException, ResourceUnavailableException
    {
        TerminateInputDocument input = TerminateInputDocument.Factory.newInstance();
        input.addNewTerminateInput();
        if ( reason != null )
        {
            input.getTerminateInput().set( reason );
        }

        client.getApplicationClient().terminate( input );
    }

    @Override
    public void destroy() throws ResourceUnknownException, ResourceUnavailableException
    {
        client.getApplicationClient().destroy();
    }

    @Override
    public AgreementPropertiesType getResourceProperties()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return client.getApplicationClient().getResourceProperties().getAgreementProperties();
    }
}
