/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.rest;

import java.net.URI;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Vector;

import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.rest.RestAgreementFactory;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.ogf.graap.wsag.client.api.AgreementClient;
import org.ogf.graap.wsag.client.api.AgreementFactoryClient;
import org.ogf.graap.wsag.client.api.NegotiationClient;
import org.ogf.graap.wsag.client.api.impl.AbstractAgreementFactoryClient;
import org.ogf.schemas.graap.wsAgreement.AgreementFactoryPropertiesType;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.CreateAgreementInputDocument;
import org.ogf.schemas.graap.wsAgreement.CreateAgreementInputType;
import org.ogf.schemas.graap.wsAgreement.CreatePendingAgreementInputDocument;
import org.ogf.schemas.graap.wsAgreement.CreatePendingAgreementInputType;
import org.ogf.schemas.graap.wsAgreement.negotiation.InitiateNegotiationInputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * Rest implementation of an agreement factory client.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class RestAgreementFactoryClient extends AbstractAgreementFactoryClient
{
    private final RestRemoteClient<RestAgreementFactory> client;

    /**
     * Constructs a new RESTful agreement factory client
     * 
     * @param epr
     *            the factory URI as endpoint reference
     * @param properties
     *            the client properties
     * @param securityProperties
     *            the security properties
     */
    public RestAgreementFactoryClient( EndpointReferenceType epr, Properties properties,
                                       ISecurityProperties securityProperties )
    {
        client =
            RestRemoteClient.getInstance( epr, properties, securityProperties, RestAgreementFactory.class );
    }

    /**
     * Constructs a new RESTful agreement factory client
     * 
     * @param uri
     *            the factory URI
     * @param properties
     *            the client properties
     * @param securityProperties
     *            the security properties
     */
    public RestAgreementFactoryClient( String uri, Properties properties,
                                       ISecurityProperties securityProperties )
    {
        client =
            RestRemoteClient.getInstance( uri, properties, securityProperties, RestAgreementFactory.class );
    }

    @Override
    public AgreementTemplateType[] getTemplates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return getFactory().getTemplates().getAgreementFactoryProperties().getTemplateArray();
    }

    @Override
    public AgreementClient createAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        //
        // instantiate the input document and set the default prefix, otherwise the xmlbeans serialization
        // fails
        //
        CreateAgreementInputDocument input = CreateAgreementInputDocument.Factory.newInstance();
        CreateAgreementInputType createAgreementInput = input.addNewCreateAgreementInput();

        createAgreementInput.addNewAgreementOffer().set( offer.getXMLObject() );
        URI response = getFactory().createAgreement( input );

        EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
        epr.addNewAddress().setStringValue( response.toASCIIString() );

        return new RestAgreementClient( epr, getSecurityProperties() );
    }

    @Override
    public AgreementClient createPendingAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        //
        // instantiate the input document and set the default prefix, otherwise the xmlbeans serialization
        // fails
        //
        CreatePendingAgreementInputDocument input = CreatePendingAgreementInputDocument.Factory.newInstance();
        CreatePendingAgreementInputType pendingAgreementInput = input.addNewCreatePendingAgreementInput();

        pendingAgreementInput.addNewAgreementOffer().set( offer.getXMLObject() );
        URI response = getFactory().createPendingAgreement( input );

        EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
        epr.addNewAddress().setStringValue( response.toASCIIString() );

        return new RestAgreementClient( epr, getSecurityProperties() );
    }

    @Override
    public AgreementClient createPendingAgreement( AgreementOffer offer, EndpointReferenceType acceptanceEPR )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        //
        // instantiate the input document and set the default prefix, otherwise the xmlbeans serialization
        // fails
        //
        CreatePendingAgreementInputDocument input = CreatePendingAgreementInputDocument.Factory.newInstance();
        CreatePendingAgreementInputType pendingAgreementInput = input.addNewCreatePendingAgreementInput();

        pendingAgreementInput.addNewAgreementOffer().set( offer.getXMLObject() );
        pendingAgreementInput.addNewAgreementAcceptanceEPR().set( acceptanceEPR );
        URI response = getFactory().createPendingAgreement( input );

        EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
        epr.addNewAddress().setStringValue( response.toASCIIString() );

        return new RestAgreementClient( epr, getSecurityProperties() );
    }

    @Override
    public NegotiationClient initiateNegotiation( NegotiationContextType context )
        throws NegotiationFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        InitiateNegotiationInputDocument input = InitiateNegotiationInputDocument.Factory.newInstance();
        input.addNewInitiateNegotiationInput();
        input.getInitiateNegotiationInput().addNewNegotiationContext().set( context );
        URI response = getFactory().initiateNegotiation( input );

        EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
        epr.addNewAddress().setStringValue( response.toASCIIString() );

        return new RestNegotiationClient( epr, new Properties(), getSecurityProperties() );
    }

    @Override
    public String getResourceId() throws ResourceUnknownException, ResourceUnavailableException
    {
        return getFactory().getResourceId();
    }

    @Override
    public AgreementFactoryClient clone() throws CloneNotSupportedException
    {
        String uri = client.getRemoteReference().getAddress().getStringValue();
        return new RestAgreementFactoryClient( uri, getProperties(), getSecurityProperties() );
    }

    @Override
    public AgreementClient[] listAgreements() throws ResourceUnknownException, ResourceUnavailableException
    {
        List<URI> agreementURIs = getFactory().listAgreements();
        List<AgreementClient> result = new Vector<AgreementClient>();

        for ( Iterator<URI> iterator = agreementURIs.iterator(); iterator.hasNext(); )
        {
            URI uri = iterator.next();
            EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
            epr.addNewAddress().setStringValue( uri.toASCIIString() );
            result.add( new RestAgreementClient( epr, getSecurityProperties() ) );
        }
        return result.toArray( new AgreementClient[result.size()] );
    }

    @Override
    public RestRemoteClient<RestAgreementFactory> getRemoteClient()
    {
        return client;
    }

    /**
     * Returns the REST agreement factory client.
     * 
     * @return the REST agreement factory client
     */
    public RestAgreementFactory getFactory()
    {
        return client.getApplicationClient();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.client.api.impl.AbstractAgreementFactoryClient#getResourceProperties()
     */
    @Override
    public AgreementFactoryPropertiesType getResourceProperties()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return getFactory().getResourceProperties().getAgreementFactoryProperties();
    }

}
