/* 
 * Copyright (c) 2012, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.rest;

import java.util.List;
import java.util.Properties;
import java.util.Vector;

import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
import org.ogf.graap.wsag.api.rest.providers.ServiceExceptionMapper;
import org.ogf.graap.wsag.api.rest.providers.URIListProvider;
import org.ogf.graap.wsag.api.rest.providers.URIProvider;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.ogf.graap.wsag.client.api.RemoteClient;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import com.ctc.wstx.api.WstxInputProperties;

/**
 * Default implementation of a RESTful WSAG4J client. Instances of the remote client implementation must
 * specify the type of the REST resource as an JAX-RS anotated interface.
 * 
 * @param <T>
 *            the type of the REST interface definition
 * 
 * @author owaeld
 * 
 */
public class RestRemoteClient<T> extends RemoteClient
{
    private final T applicationClient;

    private RestRemoteClient( EndpointReferenceType reference, Properties properties,
                              ISecurityProperties securityProperties, Class<T> interfaceClass )
    {
        super( reference, properties, securityProperties );

        //
        // Enable Woodstox API to use default namespaces (without namespace prefixes). Per default Woodstox
        // requires namespaces for all elements and attributes, otherwise serialization fails with MARSHALL
        // ERROR. By enabling this property XML documents that use default namespaces are serialized properly,
        // i.e. when XmlBeans are created by the API.
        //
        // Woodstox API implementation mentions that "the default value for 4.x is 'false', but this will most
        // likely change for 5.0 since Stax API actually specifies null to be used".
        //
        System.setProperty( WstxInputProperties.P_RETURN_NULL_FOR_DEFAULT_NAMESPACE, Boolean.toString( true ) );

        List<Object> providers = new Vector<Object>();
        providers.add( new URIProvider() );
        providers.add( new URIListProvider() );
        providers.add( new ServiceExceptionMapper() );
        applicationClient =
            JAXRSClientFactory.create( getRemoteReference().getAddress().getStringValue(), interfaceClass,
                providers );
    }

    /**
     * Returns a new instance of a REST remote client.
     * 
     * @param reference
     *            the URI of the REST resource as endpoint reference
     * @param properties
     *            the client properties
     * @param securityProperties
     *            the client security properties
     * @param interfaceClass
     *            the class of the JAX-RS annotated interface of the REST resource
     * @param <U>
     *            the JAX-RS annotated interface of the REST resource
     * 
     * @return the REST client
     */
    public static <U> RestRemoteClient<U> getInstance( EndpointReferenceType reference,
                                                       Properties properties,
                                                       ISecurityProperties securityProperties,
                                                       Class<U> interfaceClass )
    {
        return new RestRemoteClient<U>( reference, properties, securityProperties, interfaceClass );
    }

    /**
     * Returns a new instance of a REST remote client.
     * 
     * @param uri
     *            the URI of the REST resource
     * @param properties
     *            the client properties
     * @param securityProperties
     *            the client security properties
     * @param interfaceClass
     *            the class of the JAX-RS annotated interface of the REST resource
     * @param <U>
     *            the JAX-RS annotated interface of the REST resource
     * 
     * @return the REST client
     */
    public static <U> RestRemoteClient<U> getInstance( String uri, Properties properties,
                                                       ISecurityProperties securityProperties,
                                                       Class<U> interfaceClass )
    {
        EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
        epr.addNewAddress().setStringValue( uri );
        return getInstance( epr, properties, securityProperties, interfaceClass );
    }

    /**
     * Returns the REST client instance.
     * 
     * @return the REST client instance.
     */
    public synchronized T getApplicationClient()
    {
        return applicationClient;
    }

}
